﻿using System;
using System.Linq;
using System.Net;
using System.Text;
using System.Windows;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using OAuth;

//oAuth 관리 객체에 대한 오픈 소스 : https://github.com/danielcrenna/oauth
//NuGet package OAuth로 검색 OAuth, An implementation of OAuth 1.0a for .Net 설치
//핵심 : 각 단계마다 oAuth관리 객체에 필요한 데이터를 넣고 .GetAuthorizationHeader()를 호출해서 
//       헤더 정보를 만든 후 request 객체의 해더에 방금 만든 정보를 추가하고 호출
namespace DaumPhoneApp
{
    public partial class MainPage : PhoneApplicationPage
    {
        //daum oAuth : http://dna.daum.net/apis/oauth
        //컨슈머 등록 : https://apis.daum.net/oauth/consumer/input
        //컨슈머 키, 시크릿
        const string consumerKey = "";
        const string consumerSecret = "";

        //url들
        string requestUrl = "https://apis.daum.net/oauth/requestToken";
        string userAuthorizeUrl = "https://apis.daum.net/oauth/authorize";
        string accessUrl = "https://apis.daum.net/oauth/accessToken";
        string callbackUrl = "oob";

        //oAuth관리 객체
        OAuthRequest client;

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            //oAuth 관리 객체 생성
            client = new OAuthRequest 
            {
                Method = "GET",
                ConsumerKey = consumerKey,
                ConsumerSecret = consumerSecret,
                CallbackUrl = callbackUrl,
                SignatureMethod = OAuthSignatureMethod.HmacSha1,
                Realm = "https://apis.daum.net/oauth",
                Version = "1.0"
            };
        }

        //Request Token 받아오기
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            //request token
            client.RequestUrl = requestUrl;
            var auth = client.GetAuthorizationHeader();

            var request = (HttpWebRequest)WebRequest.Create(client.RequestUrl);
            request.Headers["Authorization"] = auth;
            request.BeginGetResponse(new AsyncCallback(callBack), request);
        }

        private void callBack(IAsyncResult asyncResult)
        {
            //결과 처리
            var request = (HttpWebRequest)asyncResult.AsyncState;
            var response = request.EndGetResponse(asyncResult);
            var content = response.GetResponseStream();
            int length = Convert.ToInt32(content.Length);
            byte[] bytes = new byte[length];
            content.Read(bytes, 0, length);

            Encoding enc = Encoding.UTF8;
            string myString = enc.GetString(bytes, 0, length);
            //받아온 토큰을 client에 입력
            var splite = myString.Split('&');
            var token = from kkk in splite
                        let sp = kkk.Split('=')
                        select new { key = sp[0], value = sp[1] };
            client.Token = token.FirstOrDefault(p => p.key == "oauth_token").value;
            client.TokenSecret = token.FirstOrDefault(p => p.key == "oauth_token_secret").value;

            //사용자 인증 페이지 출력
            WebBrowserTask webTask = new WebBrowserTask();
            var uri = new Uri(userAuthorizeUrl + "?oauth_callback=oob&oauth_token=" + client.Token, UriKind.Absolute);
            webTask.Uri = uri;
            webTask.Show();
        }

        private void Button2_Click(object sender, RoutedEventArgs e)
        {
            //사용자 인증 번호 입력 후 accessToken 받기 위해서 다시 호출
            client.RequestUrl = accessUrl;
            client.Verifier = txtVerifier.Text;
            var auth = client.GetAuthorizationHeader();

            var request = (HttpWebRequest)WebRequest.Create(client.RequestUrl);
            request.Headers["Authorization"] = auth;
            request.BeginGetResponse(new AsyncCallback(callBack2), request);
        }

        private void callBack2(IAsyncResult asyncResult)
        {
            //결과 처리
            var request = (HttpWebRequest)asyncResult.AsyncState;
            var response = request.EndGetResponse(asyncResult);
            var content = response.GetResponseStream();
            int length = Convert.ToInt32(content.Length);
            byte[] bytes = new byte[length];
            content.Read(bytes, 0, length);

            Encoding enc = Encoding.UTF8;
            string myString = enc.GetString(bytes, 0, length);
            //accessToken 을 다시 client에 입력
            var splite = myString.Split('&');
            var token = from kkk in splite
                        let sp = kkk.Split('=')
                        select new { key = sp[0], value = sp[1] };
            client.Token = token.FirstOrDefault(p => p.key == "oauth_token").value;
            client.TokenSecret = token.FirstOrDefault(p => p.key == "oauth_token_secret").value;

            //버튼 활성화
            Dispatcher.BeginInvoke(() => 
            {
                btnGetCafeList.IsEnabled = true;
            });
       }

        private void Button3_Click(object sender, RoutedEventArgs e)
        {   
            //카페 목록 조회
            //client.RequestUrl = "http://apis.daum.net/cafe/alimis.xml";
            client.RequestUrl = "http://apis.daum.net/cafe/favorite_cafes.xml";
            var auth = client.GetAuthorizationHeader();
            var request = (HttpWebRequest)WebRequest.Create(client.RequestUrl);
            request.Headers["Authorization"] = auth;
            request.BeginGetResponse(new AsyncCallback(callBack3), request);
        }

        private void callBack3(IAsyncResult asyncResult)
        {
            //결과 처리
            var request = (HttpWebRequest)asyncResult.AsyncState;
            var response = request.EndGetResponse(asyncResult);
            var content = response.GetResponseStream();
            int length = Convert.ToInt32(content.Length);
            byte[] bytes = new byte[length];
            content.Read(bytes, 0, length);

            Encoding enc = Encoding.UTF8;
            string cafeList = enc.GetString(bytes, 0, length);

            //카페 목록 출력
            Dispatcher.BeginInvoke(() =>
            {
                tbList.Text = cafeList;
            });
        }
    }
}